/*
   ┏━━━┓╱╱╱╱╱┏┓╱╱┏┓╱╱╱╱╱┏┓╱╱   
   ┃┏━┓┃╱╱╱╱╱┃┃╱╱┃┃╱╱╱╱╱┃┃╱╱╱  
   ┃┗━┛┣┓┏┳━━┫┃┏┓┃┃╱╱┏━━┫┗━┳━━┓
   ┃┏━━┫┃┃┃┏┓┃┗┛┃┃┃╱┏┫┏┓┃┏┓┃━━┫
   ┃┃╱╱┃┗┛┃┃┃┃┏┓┃┃┗━┛┃┏┓┃┗┛┣━━┃
   ┗┛╱╱┗━━┻┛┗┻┛┗┛┗━━━┻┛┗┻━━┻━━┛
    ━━━━━━━━━━━━━━━━━━━━━━━━━━ 

	Copyright (c) 2022 Punk Labs LLC

	This file is part of OneTrick CRYPTID

	OneTrick CRYPTID is free software: you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by the Free
	Software Foundation, either version 3 of the License, or (at your option)
	any later version.

	OneTrick CRYPTID is distributed in the hope that it will be useful, but WITHOUT
	ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
	FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
	more details.

	You should have received a copy of the GNU General Public License along with
	OneTrick CRYPTID.  If not, see <http://www.gnu.org/licenses/>.
*/

import("stdfaust.lib");

import("dx808.lib");

pluginTriggerAndVelocity = nentry("Trigger", 0, 0, 1, 0.01);

//pluginTrigger = (pluginTriggerAndVelocity>0);
pluginTrigger = pluginTriggerAndVelocity > pluginTriggerAndVelocity';

pluginVelocity = pluginTriggerAndVelocity : ba.sAndH(pluginTrigger);
pluginChokeGate = nentry("Choke", 0, 0, 1, 0.01);
pluginWakeUpGate = checkbox("WakeUp");

pluginChokeSeconds = 20/1000;
pluginChokeEnv = en.ar(0, pluginChokeSeconds, pluginChokeGate) * pluginChokeGate + (1-pluginChokeGate);

//pluginTriggerIndicator = en.ar(0, 1.0, pluginTrigger) : hbargraph("Indicator",-60,0); // Sample-rate, CPU waste
//attachTriggerIndicator = attach(_, pluginTriggerIndicator);

smoothParam(gate) = si.smooth(0.9995 * (pluginWakeUpGate<=pluginWakeUpGate')); // Skip smoothing when waking up

midikey = hslider("key", 36, 0, 127, 1);
midigain = hslider("gain",0,0,1,0.01);
midigate = button("gate");
midinote = environment {
	key = midikey;
	freq = ba.midikey2hz(midikey);
	velocity = max(midigain, pluginVelocity) * 127;
	
	gate = midigate;			//DX-7
	trigger = pluginTrigger;	//DX-808
};



stereo(node) = (node, node);

dcFilter = fi.highpass(1, 20); // Not strictly necessary, remove as optimization

// A simple tanh with an averaging of the samples to dull it like aa.tanh1.
// Originally, we used aa.tanh1, but that introduced clicking artifacts.
// This should keep the old sound without clicking:
tanh_saturation_warm(x) = ma.tanh((x+x')/2);

// #################### VOICES ####################
voiceBlendAmount = hslider("[0]Voice_Blend[export:Voice][unit:%]", 0, 0, 100, 0.01) * 0.01;

voice(p) = dx7_patch(p, midinote);
voiceBlend(a, b) = it.interpolate_linear(voiceBlendAmount, a, b);
voiceGain(adjustGain) = ba.db2linear(max(-100, hslider("[1]Voice_Gain[export:Gain][unit:dB]", 0, -100, 6, 0.1) + adjustGain));
voicePan = hslider("[2]Voice_Pan[export:Pan][unit:%]", 0, -100, 100, 0.01) * 0.01;
voiceFX = hslider("[3]Voice_FX[export:FX Send][unit:%]", 0, 0, 100, 0.01) * 0.01;


stereoPanner(pan) = *(min(1,1-pan)), *(min(1,pan+1));
// Adds stereo latency to cheaply simulate microphone placement
stereoSeparation(pan) = result with {
	earSpacing = 500/1000; //Should be 20cm, but 500cm sounds better
	maxDelay = earSpacing / pm.speedOfSound;
	maxDelaySamples = maxDelay * ma.SR;
	panLeft = max(0, -pan);
	panRight = max(0, pan);
	delayLeftSamples = panRight * maxDelaySamples;
	delayRightSamples = panLeft * maxDelaySamples;
	result = (de.fdelay(maxDelaySamples, delayLeftSamples), de.fdelay(maxDelaySamples, delayRightSamples));
};


voiceReverbSend = hslider("[8]Voice_Reverb[export:Reverb][unit:%]", 20, 0, 100, 0.01) * 0.01;

//voiceOut(adjustGain) = dcFilter : *(voiceGain(adjustGain)) <: stereoPanner(voicePan);// : stereoSeparation(voicePan); //Stereo Separation is probably overkill..
voiceOut(adjustGain) = dcFilter : *(voiceGain(adjustGain)) <: si.bus(3) : (stereoPanner(voicePan), (_*voiceReverbSend));

voiceChoke = *(pluginChokeEnv);

voiceTranspose24 = hslider("[4]Voice_Transpose[export:Transpose][unit:st]",0,-24,24,0.01);
voiceTranspose12 = hslider("[4]Voice_Transpose[export:Transpose][unit:st]",0,-12,12,0.01);
voiceTranspose6 = hslider("[4]Voice_Transpose[export:Transpose][unit:st]",0,-6,6,0.01);

voiceTone = hslider("[7]Voice_Tone[export:Tone][unit:%]", 90, 0, 100, 0.01) * 0.01 * 0.99; // 99% to avoid click at 100

// Q: (0.707 - 25.0)... Docs say 4.0 can self oscillate, but it doesn't seem to
// Higher Q values seem to reduce bass...
// Let's fade reso in as the tone goes down. Why not!
voiceQ = it.interpolate_linear(voiceTone, 3.0, 0.707);
voiceToneFilter = ve.moogLadder(voiceTone, voiceQ); //0.707); 


voiceDecay = hslider("[5]Voice_Decay[export:Decay][unit:%]", 50, 0, 100, 0.01) * 0.01;

processTest = voice(component("patches/test.lib").patch) : voiceOut(0); // Calibrated to Dexed for testing

boostGlobal = 12;

// #################### KICK ####################
kickBoost = -2.5;
kickThumpy = hslider("[6]Voice_Thumpy[export:Thumpy][unit:%]", 75, 0, 100, 0.01) * 0.01;
kickPatch = component("patches/kick.lib").patch;
processKick = voice(kickPatch[
	transpose = kickPatch.transpose+voiceTranspose6-3+0.25; //Tweak to compensate for downward slide
	pitchEnv = kickPatch.pitchEnv[
		r1=it.interpolate_linear(voiceDecay, 55, 40);
	];
	op1 = kickPatch.op1[
		r2=it.interpolate_linear(voiceDecay, 55, 40);
		r3=it.interpolate_linear(voiceDecay, 55, 40);
	];
	op3 = kickPatch.op3[
		outLevel = it.interpolate_linear(ba.log2LinGain(kickThumpy), 70, 99);
		r2=it.interpolate_linear(voiceDecay, 55, 40);
		r3=it.interpolate_linear(voiceDecay, 55, 40);
	];
	op5 = kickPatch.op5[
		outLevel = it.interpolate_linear(ba.log2LinGain(kickThumpy), 70, 99);
	];
	
]) : voiceOut(boostGlobal+kickBoost);

// #################### SNARE ####################
snareBoost = -2.5;
snareSnappy = hslider("[6]Voice_Snappy[export:Snappy][unit:%]", 75, 0, 100, 0.01) * 0.01;
snarePatch = component("patches/snare.lib").patch;
processSnare = voice(snarePatch[
	transpose = snarePatch.transpose+voiceTranspose12-3;
	op4 = snarePatch.op4[
		outLevel = it.interpolate_linear(ba.log2LinGain(snareSnappy), 70, 99);
	];
]) : voiceToneFilter : voiceOut(boostGlobal+snareBoost);

// #################### RIMSHOT ####################
rimshotBoost = -11.5;
processRimshot1 = voice(component("patches/rimshot1.lib").patch) : voiceOut(boostGlobal+rimshotBoost +1.0);
processRimshot2 = voice(component("patches/rimshot2.lib").patch) : voiceOut(boostGlobal+rimshotBoost -2.5);


// #################### CLAP ####################
clapBoost = -9;
clap1Patch = component("patches/clap1.lib").patch; // Clap
processClap1 = voice(clap1Patch[
    transpose = clap1Patch.transpose+voiceTranspose24+12;
]) : voiceToneFilter : voiceOut(boostGlobal+clapBoost -2);
clap2Patch = component("patches/clap2.lib").patch;
clap2Attack = hslider("[6]Voice_Attack[export:Attack][unit:%]", 50, 0, 100, 0.01) * 0.01;
processClap2 = voice(clap2Patch[ // Maraca
    //transpose = clap2Patch.transpose+voiceTranspose24;
    op3 = clap2Patch.op3[
        r1 = it.interpolate_linear(clap2Attack, 71, 60);
    ];
    //op2.r1 = 69-57
]) : voiceToneFilter : voiceOut(boostGlobal+clapBoost +0);//[dx808Env=0;]


// #################### TOMS ####################
tomBoost = -9.5;
tomTranspose = -0.3;
tom1Patch = component("patches/tom1.lib").patch; // Tom
processTom1 = voice(tom1Patch[
	transpose = tom1Patch.transpose+voiceTranspose12+tomTranspose;
]) : voiceOut(boostGlobal+tomBoost +1);

tom2Patch = component("patches/tom2.lib").patch; // Conga
processTom2 = voice(tom2Patch[
//    dx808Env=0;
	transpose = tom2Patch.transpose+voiceTranspose12+tomTranspose;
]) : voiceOut(boostGlobal+tomBoost -2.5);

tom3Patch = component("patches/tom3.lib").patch; // Bongo
processTom3 = voice(tom3Patch[
	transpose = tom3Patch.transpose+voiceTranspose12+tomTranspose;
]) : voiceOut(boostGlobal+tomBoost -1);

tom4Patch = component("patches/tom4.lib").patch; // Log Drum
processTom4 = voice(tom4Patch[
	transpose = tom4Patch.transpose+voiceTranspose12+tomTranspose +0.5;
]) : voiceOut(boostGlobal+tomBoost -5.5);


// #################### HIHATS ####################
hihatBoost = -9;

// #################### HIHAT CLOSED ####################
hihatClosedPatch = component("patches/hihat_closed.lib").patch;
processHihatClosed = voice(hihatClosedPatch[
	op3 = hihatClosedPatch.op3[
		r2=it.interpolate_linear(voiceDecay, 65, 60); // 63, 57
	];
]) : voiceToneFilter : voiceOut(boostGlobal+hihatBoost);

// #################### HIHAT OPEN ####################
hihatOpenPatch = component("patches/hihat_open.lib").patch;
processHihatOpen = voice(hihatOpenPatch[
	op3 = hihatOpenPatch.op3[
		r2=it.interpolate_linear(voiceDecay, 63, 55);
		r3=it.interpolate_linear(voiceDecay, 56, 45);
	];
]) : voiceToneFilter : voiceChoke : voiceOut(boostGlobal+hihatBoost);

// #################### CYMBAL ####################
cymbalBoost = -6;
cymbal1Patch = component("patches/cymbal1.lib").patch; // Cymbal
processCymbal1 = voice(cymbal1Patch[
	op3 = cymbal1Patch.op3[
		r2=it.interpolate_linear(voiceDecay, 63, 50);
		r3=it.interpolate_linear(voiceDecay, 56, 33);
	];
]) : voiceToneFilter : voiceChoke : voiceOut(boostGlobal+cymbalBoost +0);
cymbal2Patch = component("patches/cymbal2.lib").patch; // Gong
processCymbal2 = voice(cymbal2Patch[
    transpose = cymbal2Patch.transpose;
    op1 = cymbal2Patch.op1[
        r2 = it.interpolate_linear(voiceDecay, 40, 26);
        r3 = it.interpolate_linear(voiceDecay, 40, 20);
    ];
]) : voiceToneFilter : voiceChoke : voiceOut(boostGlobal+cymbalBoost -7);
cymbal3Patch = component("patches/cymbal3.lib").patch; // Reverse Cymbal
processCymbal3 = voice(cymbal3Patch[
		dx808Env=0;
		op3 = cymbal3Patch.op3[
		r1=it.interpolate_linear(voiceDecay, 50, 30);//40.5);
	];
]) : voiceToneFilter : voiceChoke : voiceOut(boostGlobal+cymbalBoost +0);

// #################### COWBELL ####################
cowbellBoost = -1;
cowbell1Patch = component("patches/cowbell1.lib").patch; // Cowbell
processCowbell1 = voice(cowbell1Patch[
    transpose = cowbell1Patch.transpose+voiceTranspose12;
]) : voiceOut(boostGlobal+cowbellBoost -11);

cowbell2Patch = component("patches/cowbell2.lib").patch; // W.Block
processCowbell2 = voice(cowbell2Patch[
	transpose = cowbell2Patch.transpose+voiceTranspose12+12;
]) : voiceOut(boostGlobal+tomBoost+cowbellBoost -1);

cowbell3Patch = component("patches/cowbell3.lib").patch; // S. Whistle
processCowbell3 = voice(cowbell3Patch[
    dx808Env=0;
	transpose = cowbell3Patch.transpose+voiceTranspose12+14;
    //transpose = 5;
]) : voiceOut(boostGlobal+cowbellBoost -13);



// #################### GLOBALS ####################
globalVolume = ba.db2linear(max(-100, hslider("[0]Global_Volume[export:Volume][unit:dB]", 0, -100, 6, 0.1)));
globalSaturation = hslider("[1]Global_Saturation[export:Saturation][unit:%]", 50, 0, 100, 0.01) * 0.01;
globalReverb = hslider("[3]Global_Reverb[export:Reverb][unit:%]", 50, 0, 100, 0.01) * 0.01;
globalReverbDamp = hslider("[6]Global_ReverbDamp[export:Reverb Damp][unit:%]", 50, 0, 100, 0.01) * 0.01;
globalReverbSize = it.interpolate_linear(hslider("[7]Global_ReverbSize[export:Reverb Size][unit:%]", 25, 0, 100, 0.01) * 0.01, 0.1, 2); //(0.1-5) is a little much
globalReverbDiffusion = hslider("[8]Global_ReverbDiffusion[export:Reverb Diffusion][unit:%]", 50, 0, 100, 0.01) * 0.01;
globalReverbGate = hslider("[9]Globals_ReverbGate[export:Reverb Gate][unit:db]", -60, -60, 0, 0.01);

// Saturation
addSaturation = ef.dryWetMixer(globalSaturation, tanh_saturation_warm);

// Reverb
revTime = 0.05;//0.2; // 0.1 - 60 
revDamp = globalReverbDamp; //0 - 1
revSize = globalReverbSize;//1; //0.5 - 5
revDiffusion = globalReverbDiffusion;//0.5;
revFeedback = 0.4;
revModDepth = 0;//0.1;
revModFreq = 2;

addReverbGate = ef.gate_stereo(globalReverbGate, 0.1/1000, 100/1000, 20/1000);
rev = re.greyhole(revTime, revDamp, revSize, revDiffusion, revFeedback, revModDepth, revModFreq);
//rev = dattorro_rev_default
revFilter = stereo(fi.highpass(1, 200)); // : fi.lowpass(2, 8000)
addReverb = si.bus(2) <: (si.bus(2), (rev : revFilter : addReverbGate : stereo(*(globalReverb)) )) :> si.bus(2);
addReverbFromSend = (si.bus(2), (_ <: rev : revFilter : addReverbGate : stereo(*(globalReverb)) )) :> si.bus(2);


// Output
processOutput = addReverbFromSend : stereo(addSaturation) : stereo(*(globalVolume));


// Default App
mute = (1-checkbox("mute"));
process = processTest : (*(mute), *(mute));






